//===- LuaBindingsAction.cpp ---------------------------------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// LuaBindingsAction.cpp - Declaration of LuaBindingsAction.
//
//===----------------------------------------------------------------------===//

#include "LuaBindings.h"

namespace LuaBindings {

using namespace clang;

ASTConsumer *LuaBindingsAction::CreateASTConsumer(CompilerInstance &CI, llvm::StringRef) {
  return new LuaBindingsConsumer(CI, OutputFileName, OutputFile, IncludeFileName);
}

bool LuaBindingsAction::ParseArgs(const CompilerInstance &CI,
               const std::vector<std::string>& args) {
  for (unsigned i = 0, e = args.size(); i != e; ++i) {
    if (args[i].compare(0, 5, "-out=") == 0) {
      OutputFileName = args[i].substr(5);
    }
    else if (args[i].compare(0, 9, "-include=") == 0) {
      IncludeFileName= args[i].substr(9);
    }
    else if (args[i] == "help")
      PrintHelp(llvm::errs());
    else {
      Diagnostic &D = CI.getDiagnostics();
      unsigned DiagID = D.getCustomDiagID(
        Diagnostic::Error, "invalid argument '" + args[i] + "'");
      D.Report(DiagID);
      return false;
    }
  }

  if (OutputFileName.length() != 0) {
    if (OutputFileName.length() != 0) {
      OutputFile = new llvm::tool_output_file(OutputFileName.c_str(), ErrorInfo, 0);
      if (ErrorInfo.length() != 0) {
        Diagnostic &D = CI.getDiagnostics();
        unsigned DiagID = D.getCustomDiagID(
          Diagnostic::Error, "LuaBindings error: " + ErrorInfo);
        D.Report(DiagID);
        delete OutputFile;
        OutputFile = NULL;
      }
    }
  }
  else {
      Diagnostic &D = CI.getDiagnostics();
      unsigned DiagID = D.getCustomDiagID(
        Diagnostic::Error, "The LuaBindings plug-in requires an output file.");
      D.Report(DiagID);
      PrintHelp(llvm::errs());
      return false;
  }
  return true;
}

void LuaBindingsAction::PrintHelp(llvm::raw_ostream& ros) {
  ros << "Usage: Clang -cc1 -load (path)/LuaBindings.dll -plugin luabindings -plugin-arg-luabindings -out=(bindingsFile).cpp (inputFile).[h|cpp]\n";
}

LuaBindingsAction::LuaBindingsAction()
  : OutputFile(NULL) {
}

LuaBindingsAction::~LuaBindingsAction() {
  if (OutputFile != NULL)
    OutputFile->os().flush();
  if ((ErrorInfo.length() == 0) && (OutputFile != NULL))
    OutputFile->keep();
  delete OutputFile;
  OutputFile = NULL;
}

}
