// Function.cpp - Example file for Clang LuaBinding plugin.  This file contains the definitions of the
//     of the binding targets.

#include <stdio.h>
#include <string.h>
#include "../BindIn/Function.h"

#ifdef _WIN32
  #define STRDUP _strdup
#else
  #define STRDUP strdup
#endif

void VoidFunction()
{
  printf("void VoidFunction() called.  Returning void.\n");
  return ;
}

namespace MyFunctions {

int IntFunction()
{
  printf("int IntFunction() called.  Returning 1234.\n");
  return 1234;
}

long LongFunction()
{
  printf("long LongFunction() called.  Returning 12345678.\n");
  return 12345678;
}

short ShortFunction()
{
  printf("short ShortFunction() called.  Returning 1234.\n");
  return 1234;
}

unsigned UnsignedFunction()
{
  printf("unsigned UnsignedFunction() called.  Returning 12345.\n");
  return 12345;
}

float FloatFunction()
{
  printf("float FloatFunction() called.  Returning 1.234.\n");
  return 1.234f;
}

double DoubleFunction()
{
  printf("double DoubleFunction() called.  Returning 1.2345678.\n");
  return 1.2345678;
}

char anArray[] = "ABCD";
char *aPointer = anArray;

void *VoidPointerFunction()
{
  char *buffer = STRDUP(anArray);
  printf("void *VoidPointerFunction() called.  Returning \"ABCD\" (%p).\n", buffer);
  return buffer;
}

int *IntPointerFunction()
{
  char *buffer = STRDUP(anArray);
  printf("int *IntPointerFunction() called.  Returning \"ABCD\" (%p).\n", aPointer);
  return (int*)buffer;
}

char CharFunction()
{
  printf("char CharFunction() called.  Returning 'A'.\n");
  return 'A';
}

char *CharPointerFunction()
{
  printf("char *CharPointerFunction() called.  Returning \"ABCD\" (%p).\n", aPointer);
  return aPointer;
}

const char *ConstCharPointerFunction()
{
  printf("const char *ConstCharPointerFunction() called.  Returning \"ABCD\" (%p).\n", aPointer);
  return aPointer;
}

StructType aStruct = { 4321 };

StructType StructFunction()
{
  printf("StructType StructFunction() called.  Returning StructType { 4321 }.\n");
  return aStruct;
}

StructType *StructPointerFunction()
{
  StructType *buffer = new StructType;
  buffer->member = 4321;
  printf("StructType *StructPointerFunction() called.  Returning buffer.\n");
  return buffer;
}

ClassType aClass(2345);

ClassType ClassFunction()
{
  printf("ClassType ClassFunction() called.  Returning ClassType(2345).\n");
  return aClass;
}

ClassType *ClassPointerFunction()
{
  printf("ClassType *ClassPointerFunction() called.  Returning &aClass (%p).\n", &aClass);
  return new ClassType(2345);
}

EnumType EnumFunction()
{
  printf("EnumType EnumFunction() called.  Returning enumMember234.\n");
  return enumMember234;
}

void FunctionIntArg(int intArg1)
{
  printf("void FunctionIntArg(%d) called.  Returning void.\n", intArg1);
  return ;
}

void FunctionFloatArg(float floatArg1)
{
  printf("void FunctionFloatArg(%f) called.  Returning void.\n", floatArg1);
  return ;
}

void FunctionDoubleArg(double doubleArg1)
{
  printf("void FunctionDoubleArg(%lf) called.  Returning void.\n", doubleArg1);
  return ;
}

void FunctionIntPointerArg(int *intPointerArg1)
{
  printf("void FunctionIntPointerArg(%p) called.  Returning void.\n", intPointerArg1);
  return ;
}

void FunctionCharArg(char charArg1)
{
  printf("void FunctionCharArg(%d) called.  Returning void.\n", charArg1);
  return ;
}

void FunctionCharPointerArg(char *charPointerArg1)
{
  printf("void FunctionCharPointerArg(%s) called.  Returning void.\n", charPointerArg1);
  return ;
}

void FunctionConstCharPointerArg(const char *constCharPointerArg1)
{
  printf("void FunctionConstCharPointerArg(%s) called.  Returning void.\n", constCharPointerArg1);
  return ;
}

void FunctionStructArg(StructType structArg1)
{
  printf("void FunctionStructArg({%d}) called.  Returning void.\n", structArg1.member);
  return ;
}

void FunctionStructPointerArg(StructType *structPointerArg1)
{
  printf("void FunctionStructPointerArg(%p { %d }) called.  Returning void.\n",
    structPointerArg1, structPointerArg1->member);
  return ;
}

void FunctionClassArg(ClassType classArg1)
{
  printf("void FunctionClassArg(ClassType(%d)) called.  Returning void.\n", classArg1.member);
  return ;
}

void FunctionClassPointerArg(ClassType *classPointerArg1)
{
  printf("void FunctionClassPointerArg(%p { %d }) called.  Returning void.\n",
   classPointerArg1, classPointerArg1->member);
  return ;
}

void FunctionEnumArg(EnumType enumArg1)
{
  printf("void FunctionEnumArg(%d) called.  Returning void.\n", (int)enumArg1);
  return ;
}

} // namespace MyFunctions

namespace OtherFunctions {

void FunctionIntArgs(int intArg1, int intArg2)
{
  printf("void FunctionIntArgs(%d, %d) called.  Returning void.\n", intArg1, intArg2);
  return ;
}

void FunctionOther(int arg)
{
  printf("void FunctionOther(%d) called.  Returning void.\n", arg);
  return ;
}

} // namespace MyFunctions
