// bind.cpp - Binding example code.

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern "C" {
#include "lua.h"
#include "lauxlib.h"
#include "lualib.h"
}

class Account {
  public:
    Account(double balance)      { m_Balance = balance; }
    void Deposit(double amount)  { m_Balance += amount; }
    void Withdraw(double amount) { m_Balance -= amount; }
    double Balance(void)         { return m_Balance; }
  private:
    double m_Balance;
};

class LuaBind_Account {
  static const char className[];
  static const luaL_reg methods[];
  static int Create(lua_State *L) {
    double balance = luaL_checknumber(L, 1);
    Account *self = new Account(balance);
    *(void **)lua_newuserdata(L, sizeof(void *)) = self;
    luaL_getmetatable(L, className);
    lua_setmetatable(L, -2);
    return 1;
  }
  static Account *Check(lua_State *L, int narg) {
    luaL_checktype(L, narg, LUA_TUSERDATA);
    void *self = luaL_checkudata(L, narg, className);
    if (!self)
      luaL_typerror(L, narg, className);
    return *(Account**)self;
  }
  static int Deposit(lua_State *L) {
    Account *self = Check(L, 1);
    double amount = luaL_checknumber(L, 2);
    self->Deposit(amount);
    return 0;
  }
  static int Withdraw(lua_State *L) {
    Account *self = Check(L, 1);
    double amount = luaL_checknumber(L, 2);
    self->Withdraw(amount);
    return 0;
  }
  static int Balance(lua_State *L) {
    Account *self = Check(L, 1);
    double Balance = self->Balance();
    lua_pushnumber(L, Balance);
    return 1;
  }
  static int Dispose(lua_State *L) {
    Account *self = (Account*)(*(void **)(lua_touserdata(L, 1)));
    delete self;
    return 0;
  }
public:
  static void Register(lua_State* L) {
    lua_newtable(L);
    int methodtable = lua_gettop(L);
    luaL_newmetatable(L, className);
    int metatable   = lua_gettop(L);
    lua_pushliteral(L, "__metatable");
    lua_pushvalue(L, methodtable);
    lua_settable(L, metatable);
    lua_pushliteral(L, "__index");
    lua_pushvalue(L, methodtable);
    lua_settable(L, metatable);
    lua_pushliteral(L, "__gc");
    lua_pushcfunction(L, Dispose);
    lua_settable(L, metatable);
    lua_pop(L, 1);
    luaL_register(L, 0, methods);
    lua_pop(L, 1);
    lua_register(L, className, Create);
  }
};
const char LuaBind_Account::className[] = "Account";
const luaL_reg LuaBind_Account::methods[] = {
  {"Deposit", LuaBind_Account::Deposit},
  {"Withdraw", LuaBind_Account::Withdraw},
  {"Balance", LuaBind_Account::Balance},
  {0,0}
};

int TestFunction(const char *argString) {
	printf("I have been called!  argString: %s\n", argString);
	return 1234;
}

  // Function binding.
class LuaBind_Functions
{
  static const luaL_reg methods[];
	static int TestFunction(lua_State* L) {
		const char* argString = luaL_checkstring(L, 1);
    int returnValue = ::TestFunction(argString);
		lua_pushnumber(L, returnValue);
		return 1;
	}
public:
    static void Register(lua_State* L) {
		  luaL_register(L, "other", methods);
    }
};
const luaL_reg LuaBind_Functions::methods[] = {
  {"TestFunction", LuaBind_Functions::TestFunction},
	{NULL, NULL}
};

int bind_open(lua_State *L)
{
	LuaBind_Account::Register(L);
	LuaBind_Functions::Register(L);
	return 1;
}
